@@ -1,29 +1,30 @@
 get_objects <- function(type, selection = NULL, sample_prior = F, design = NULL, sampler = NULL,
-                        prior = NULL, stage = 'sample', N = 1e5, ...){
+                        prior = NULL, stage = 'sample', N = 1e5, map = FALSE, ...){
+  selection <- map_selecter(map, selection)
   return_prior <- ifelse(is.null(sampler), TRUE, FALSE)
   if(type %in% c("standard", "blocked", "diagonal")){
-    out <- get_objects_standard(selection, sample_prior, return_prior, design, prior, stage, N = N,
-                                sampler, ...)
+    out <- get_objects_standard(selection, sample_prior, return_prior, design, prior, stage = stage, N = N,
+                                sampler = sampler, ...)
   }
   else if(type == "single"){
-    out <- get_objects_single(selection, sample_prior, return_prior, design, prior, stage, N = N,
-                                sampler, ...)
+    out <- get_objects_single(selection, sample_prior, return_prior, design, prior, stage = stage, N = N,
+                                sampler = sampler, ...)
   }
   else if(type == "diagonal-gamma"){
-    out <- get_objects_diag_gamma(selection, sample_prior, return_prior, design, prior, stage, N = N,
-                            sampler, ...)
+    out <- get_objects_diag_gamma(selection, sample_prior, return_prior, design, prior, stage = stage, N = N,
+                            sampler = sampler, ...)
   }
   else if(type == "infnt_factor"){
-    out <- get_objects_infnt_factor(selection, sample_prior, return_prior, design, prior, stage, N = N,
-                               sampler, ...)
+    out <- get_objects_infnt_factor(selection, sample_prior, return_prior, design, prior, stage = stage, N = N,
+                               sampler = sampler, ...)
   }
   else if(type == "factor"){
-    out <- get_objects_factor(selection, sample_prior, return_prior, design, prior, stage, N = N,
-                                    sampler,...)
+    out <- get_objects_factor(selection, sample_prior, return_prior, design, prior, stage = stage, N = N,
+                                    sampler = sampler,...)
   }
   else if(type == "SEM"){
-    out <- get_objects_SEM(selection, sample_prior, return_prior, design, prior, stage, N = N,
-                              sampler,...)
+    out <- get_objects_SEM(selection, sample_prior, return_prior, design, prior, stage = stage, N = N,
+                              sampler = sampler,...)
   }
   else{
     stop("make sure type is supported!")
@@ -118,8 +119,11 @@ get_objects_diag_gamma <- function(selection, sample_prior, return_prior, design
 get_objects_standard <- function(selection, sample_prior, return_prior, design = NULL,
                                  prior = NULL, stage = 'sample', N = 1e5, sampler = NULL, ...){
   acc_selection <- c("mu", "sigma2", "beta", "covariance", "correlation", "alpha", "Sigma", "LL")
+  group_design <- sampler[[1]]$group_designs
+  if(!is.null(attr(prior, "group_design"))) group_design <- attr(prior, "group_design")
+  dots <- add_defaults(list(...), group_design = group_design)
   if(return_prior & !sample_prior){
-    if(is.null(list(...)$return_info)) prior$prior <- get_prior_standard(design = design, sample = F, prior = prior, group_design = list(...)$group_design)
+    if(is.null(dots$return_info)) prior$prior <- get_prior_standard(design = design, sample = F, prior = prior, group_design = dots$group_design)
     prior$descriptions <- list(
       theta_mu_mean = "mean of the group-level mean prior",
       theta_mu_var = "variance of the group-level mean prior",
@@ -134,23 +138,22 @@ get_objects_standard <- function(selection, sample_prior, return_prior, design =
       mu = "Group-level mean",
       Sigma = 'Group-level covariance matrix'
     )
-    if(!is.null(list(...)$return_info)) return(prior[c("types", "type_descriptions", "descriptions")])
+    if(!is.null(dots$return_info)) return(prior[c("types", "type_descriptions", "descriptions")])
     prior$prior <- add_prior_names(prior$prior, design, ...)
     return(prior)
   } else{
     if(!selection %in% acc_selection) stop(paste0("selection must be in : ", paste(acc_selection, collapse = ", ")))
     if(sample_prior){
-      dots <- list(...)
       if(!is.null(sampler$par_groups)){
         dots$par_groups <- sampler$par_groups
         dots$group_design <- sampler$group_design
       }
       if(selection == "alpha" & !is.null(sampler)){
-        mu <- get_pars(sampler, selection = "mu", stage = stage, map = FALSE, return_mcmc = FALSE, merge_chains = TRUE, ...)
+        mu <- get_pars(sampler, selection = "beta", stage = stage, map = FALSE, return_mcmc = FALSE, merge_chains = TRUE, ...)
         var <- get_pars(sampler, selection = "Sigma", stage = stage, map = FALSE, return_mcmc = FALSE, merge_chains = TRUE,
                         remove_dup = FALSE, remove_constants = FALSE, ...)
         sub_names <- names(sampler[[1]]$data)
-        sampler <- list(list(samples =  list(alpha = get_alphas(mu, var, sub_names))))
+        sampler <- list(list(samples =  list(alpha = get_alphas(mu, var, sub_names, group_design = group_design))))
       } else{
         sampler <- list(list(samples = do.call(get_prior_standard, c(list(prior = prior, design = design, selection = selection,N = N),
                                                                      fix_dots(dots, get_prior_standard)))))
@@ -374,9 +377,9 @@ get_objects_infnt_factor <- function(selection, sample_prior, return_prior, desi
 
 get_objects_SEM <- function(selection, sample_prior, return_prior, design = NULL,
                                prior = NULL, stage = 'sample', N = 1e5, sampler = NULL, ...){
-  acc_selection <- c("mu", "sigma2", "covariance", "alpha", "correlation", "Sigma", "loadings", "residuals",
-                     "factor_residuals", "regressors", "factor_regressors", "structural_regressors",
-                     "mu_implied", "LL")
+  acc_selection <- c("mu", "sigma2", "covariance", "alpha", "correlation", "Sigma",
+                     "std_loadings", "loadings", "residuals","factor_residuals", "regressors",
+                     "factor_regressors", "structural_regressors","mu_implied", "LL")
   if(return_prior & !sample_prior){
     if(is.null(list(...)$return_info)) prior$prior <- do.call(get_prior_SEM, c(list(design = design, sample = F, prior = prior), fix_dots(list(...), get_prior_SEM)))
     prior$descriptions <- list(
@@ -424,9 +427,7 @@ get_objects_SEM <- function(selection, sample_prior, return_prior, design = NULL
       } else{
         dots <- list(...)
         if(!is.null(sampler)){
-          dots <- add_defaults(dots, K_mat = attr(sampler[[1]], "K_mat"), B_mat = attr(sampler[[1]], "B_mat"),
-                               covariates = sampler[[1]]$covariates,
-                               Lambda_mat = attr(sampler[[1]], "Lambda_mat"), G_mat = attr(sampler[[1]], "G_mat"))
+          dots <- add_defaults(dots, sem_settings = sampler[[1]]$sem_settings)
         }
         sampler <- list(list(samples = do.call(get_prior_SEM,
                                                c(list(prior = prior, design = design,
@@ -438,13 +439,7 @@ get_objects_SEM <- function(selection, sample_prior, return_prior, design = NULL
     }
     idx <- get_idx(sampler, stage)
     if(selection == "residuals"){
-      return(lapply(sampler, FUN = function(x){
-        resids <- x$samples$epsilon_inv[,,idx]
-        for(i in 1:dim(resids)[3]){
-          resids[,,i] <- solve(resids[,,i])
-        }
-        return(resids)
-      }))
+      return(lapply(sampler, FUN = function(x) return(1/x$samples$epsilon_inv[,idx])))
     }
     if(selection == "factor_residuals"){
       return(lapply(sampler, FUN = function(x){
@@ -470,6 +465,11 @@ get_objects_SEM <- function(selection, sample_prior, return_prior, design = NULL
     if(selection == "mu_implied"){
       return(lapply(sampler, FUN = get_mu_implied, idx))
     }
+    if(selection == "std_loadings"){
+      return(lapply(sampler, FUN = function(x) standardize_loadings_SEM(x$samples$lambda[,,idx, drop = F],
+                                                                        x$samples$theta_var[,,idx, drop = F],
+                                                                        x$samples$delta_inv[,,idx, drop = F])))
+    }
     return(get_base(sampler, idx, selection))
   }
 }
@@ -508,22 +508,39 @@ get_base <- function(sampler, idx, selection){
     return(lapply(sampler, FUN = function(x) return(x$samples$theta_var[,,idx, drop = F])))
   }
   else if(selection == "correlation"){
-    return(lapply(sampler, FUN = function(x) return(
+    return(suppressWarnings(lapply(sampler, FUN = function(x) return(
       array(apply(x$samples$theta_var[,,idx],3,cov2cor),dim=dim(x$samples$theta_var[,,idx, drop = F]),
-            dimnames=dimnames(x$samples$theta_var)))))
+            dimnames=dimnames(x$samples$theta_var))))))
   }
 }
 
 
-get_alphas <- function(mu, var, sub_names, N = ncol(mu)){
-  n_pars <- nrow(mu)
-  alpha <- array(NA_real_, dim = c(n_pars, length(sub_names), N))
-  for(i in 1:N){
-    alpha[,,i] <- t(rmvnorm(1, mu[,i], var[,,i]))
+get_alphas <- function(mu, var, sub_names = NULL, N = ncol(mu),
+                       group_design = NULL){
+
+  if (is.null(sub_names)) sub_names <- "alpha"
+  n_subs <- length(sub_names)
+
+  if (is.null(group_design)) {
+    # Simple mode: mu is (p x N), same mean for all subjects within draw i
+    p <- nrow(mu)
+    alpha <- array(NA_real_, dim = c(p, n_subs, N))
+    for (i in 1:N) {
+      alpha[,,i] <- t(rmvnorm(n_subs, mu[, i], var[,, i]))
+    }
+    rownames(alpha) <- rownames(mu)
+
+  } else {
+    # Design mode: mu is (M x N) where M = sum_k m_k (intercept + effects per param)
+    # Build subject-specific means, then draw per subject
+    group_design <- add_group_design(rownames(var), group_design, n_subs)
+    p <- length(group_design)
+    alpha <- draw_alpha_from_design(group_design, mu, var)
+    dimnames(alpha) <- list(names(group_design),sub_names,NULL)
   }
-  rownames(alpha) <- rownames(mu)
+
   colnames(alpha) <- sub_names
-  return(alpha)
+  alpha
 }
 
 
