@@ -1,4 +1,16 @@
 #### Staircase ----
+#' Assign Stop-Signal Delays (SSDs) to trials
+#'
+#' @description
+#' This function assigns stop-signal delays (SSDs) to trials based on specified probabilities.
+#'
+#' @param d A data frame containing trial data with a response factor column 'lR'
+#' @param SSD A vector of stop-signal delays to be assigned to trials
+#' @param pSSD A vector of probabilities for each SSD value. If length is one less than SSD,
+#'             the remaining probability is calculated automatically. Default is 0.25.
+#'
+#' @return A vector of SSDs with the same length as the number of rows in 'd'.
+#'         Trials without a stop signal are assigned Inf.
 SSD_function <- function(d,SSD=NA,pSSD=.25) {
   if (sum(pSSD)>1) stop("pSSD sum cannot exceed 1.")
   if (length(pSSD)==length(SSD)-1) pSSD <- c(pSSD,1-sum(pSSD))
@@ -510,6 +522,36 @@ pstopEXG <- function(parstop,n_acc,upper=Inf,
 # }
 
 
+
+#### SSexG Model list ----
+
+#' Stop-signal ex-Gaussian race
+#'
+#' Model file to estimate the ex-Gaussian race model for Stop-Signal data
+#'
+#' Model files are almost exclusively used in `design()`.
+#'
+#' @details
+#'
+#' Default values are used for all parameters that are not explicitly listed in the `formula`
+#' argument of `design()`.They can also be accessed with `SSexG()$p_types`.
+#'
+#' @details
+#' | **Parameter** | **Transform** | **Natural scale** | **Default**   | **Mapping**                    | **Interpretation**                                            |
+#' |-----------|-----------|---------------|-----------|----------------------------|-----------------------------------------------------------|
+#' | *mu*       | log         | \[0, Inf\]     | log(.4)         |                            | mu parameter of ex-Gaussian go finishing time distribution              |
+#' | *sigma*       | log       | \[0, Inf\]        | log(.05)    |                            | sigma parameter of ex-Gaussian go finishing time distribution                                        |
+#' | *tau*      | log       | \[0, Inf\]        | log(.1)    |                            | tau parameter of ex-Gaussian go finishing time distribution                                          |
+#' | *muS*       | log       | \[0, Inf\]        | log(.3)    |                            | mu parameter of ex-Gaussian stopping finishing time distribution           |
+#' | *sigmaS*       | log    | \[0, Inf\]        | log(.025)|                   | sigma parameter of ex-Gaussian stopping finishing time distribution                              |
+#' | *tauS*      | log    | \[0, Inf\]        | log(.05)  |  | tau parameter of ex-Gaussian stopping finishing time distribution       |
+#' | *tf*      | probit       | \[0, 1\]        | qnorm(0)    |                            | Trigger failure probability           |
+#' | *gf*     | probit       | \[0, 1\]        | qnorm(0)    |                            | Go failure probability    |
+#'
+#'
+#'
+#' @return A model list with all the necessary functions to sample
+#' @export
 SSexG <- function() {
   list(
     type="RACE",
@@ -565,7 +607,7 @@ rSShybrid <- function(data,pars,ok=rep(TRUE,dim(pars)[1]))
   # NB1: Go failures will only apply to accumulators where lI = TRUE
   #      and can still have a stop-triggered response on a go-failure trial.
 {
-  lR <- data$lR # For Michelle as an example
+  lR <- data$lR
   pars[,c("A","B","v")] <- pars[,c("A","B","v")]/pars[ok,"s"]
 
   nacc <- length(levels(lR))   # Does not include stop runner
@@ -743,6 +785,19 @@ pstopHybrid <- function(parstop,n_acc,upper=Inf,
 
 #### RDEX model list ----
 
+#' Stop-signal Hybrid (RDM go, ExGaussian stop) race
+#'
+#' Model file to estimate the Hybrid race model for Stop-Signal data
+#'
+#' Model files are almost exclusively used in `design()`.
+#'
+#' @details
+#'
+#' Default values are used for all parameters that are not explicitly listed in the `formula`
+#' argument of `design()`.They can also be accessed with `SShybrid()$p_types`.
+#'
+#' @return A model list with all the necessary functions to sample
+#' @export
 SShybrid <- function() {
   list(
     type="RACE",
