@@ -54,7 +54,8 @@ pmwgs <- function(dadm, type, pars = NULL, prior = NULL,
 }
 
 init <- function(pmwgs, start_mu = NULL, start_var = NULL,
-                 verbose = FALSE, particles = 1000, n_cores = 1) {
+                 verbose = FALSE, particles = 1000,
+                 n_cores = 1, r_cores = 1) {
   # Gets starting points for the mcmc process
   # If no starting point for group mean just use zeros
   type <- pmwgs$type
@@ -75,7 +76,7 @@ init <- function(pmwgs, start_mu = NULL, start_var = NULL,
   proposals <- parallel::mclapply(X=1:pmwgs$n_subjects,FUN=start_proposals,
                                   parameters = startpoints_comb, n_particles = particles,
                                   pmwgs = pmwgs, type = type,
-                                  mc.cores = n_cores)
+                                  mc.cores = n_cores, r_cores = r_cores)
   proposals <- array(unlist(proposals), dim = c(pmwgs$n_pars + 1, pmwgs$n_subjects))
 
   # Sample the mixture variables' initial values.
@@ -98,6 +99,7 @@ init <- function(pmwgs, start_mu = NULL, start_var = NULL,
 #' @param cores_per_chain An integer. How many cores to use per chain. Parallelizes across participant calculations.
 #' @param cores_for_chains An integer. How many cores to use to parallelize across chains. Default is the number of chains.
 #' @param particles An integer. Number of starting values
+#' @param ... optional additional arguments
 #'
 #' @return An emc object
 #' @examples \donttest{
@@ -106,7 +108,7 @@ init <- function(pmwgs, start_mu = NULL, start_var = NULL,
 #'                            formula =list(v~0+S,a~E, t0~1, s~1),
 #'                            constants=c(s=log(1)))
 #'
-#' DDMaE <- make_emc(forstmann, design_DDMaE)
+#' DDMaE <- make_emc(forstmann, design_DDMaE, compress = FALSE)
 #' # set up our mean starting points (same used across subjects).
 #' mu <- c(v_Sleft=-2,v_Sright=2,a=log(1),a_Eneutral=log(1.5),a_Eaccuracy=log(2),
 #'        t0=log(.2))
@@ -115,28 +117,30 @@ init <- function(pmwgs, start_mu = NULL, start_var = NULL,
 #' # Initialize chains, 4 cores per chain, and parallelizing across our 3 chains as well
 #' # so 4*3 cores used.
 #' DDMaE <- init_chains(DDMaE, start_mu = mu, start_var = var,
-#'                      cores_per_chain = 1, cores_for_chains = 1, particles = 10)
+#'                      cores_per_chain = 1, cores_for_chains = 1, particles = 3)
 #' # Afterwards we can just use fit
 #' # DDMaE <- fit(DDMaE, cores_per_chain = 4)
 #' }
 #' @export
 init_chains <- function(emc, start_mu = NULL, start_var = NULL, particles = 1000,
-                        cores_per_chain=1,cores_for_chains = length(emc))
+                        cores_per_chain=1,cores_for_chains = length(emc),
+                        ...)
 {
+  dots <- add_defaults(list(...),r_cores=1)
   emc <- mclapply(emc,init,start_mu = start_mu, start_var = start_var,
-           verbose = FALSE, particles = particles,
+           verbose = FALSE, particles = particles,r_cores=dots$r_cores,
            n_cores = cores_per_chain, mc.cores=cores_for_chains)
   class(emc) <- "emc"
   return(emc)
 }
 
-start_proposals <- function(s, parameters, n_particles, pmwgs, type){
+start_proposals <- function(s, parameters, n_particles, pmwgs, type, r_cores = 1){
   #Draw the first start point
   group_pars <- get_group_level(parameters, s, type)
   proposals <- particle_draws(n_particles, group_pars$mu, group_pars$var)
   colnames(proposals) <- rownames(pmwgs$samples$alpha) # preserve par names
   lw <- calc_ll_manager(proposals, dadm = pmwgs$data[[which(pmwgs$subjects == s)]],
-                        model = pmwgs$model)
+                        model = pmwgs$model, r_cores = r_cores)
   weight <- exp(lw - max(lw))
   idx <- sample(x = n_particles, size = 1, prob = weight)
   return(list(proposal = proposals[idx,], ll = lw[idx]))
@@ -185,7 +189,8 @@ run_stage <- function(pmwgs,
                       n_cores = 1,
                       tune = NULL,
                       verbose = TRUE,
-                      verboseProgress = TRUE) {
+                      verboseProgress = TRUE,
+                      r_cores = 1) {
   # Set defaults for NULL values
   # Set necessary local variables
   # Set stable (fixed) new_sample argument for this run
@@ -252,7 +257,7 @@ run_stage <- function(pmwgs,
                                     MoreArgs = list(pars_comb, pmwgs$model, stage,
                                                     pmwgs$type,
                                                     tune),
-                                    mc.cores =n_cores)
+                                    mc.cores =n_cores, r_cores = r_cores)
     pm_settings <- proposals[3,]
     proposals <- array(unlist(proposals[1:2,]), dim = c(pmwgs$n_pars + 1, pmwgs$n_subjects))
 
@@ -270,7 +275,7 @@ new_particle <- function (s, data, pm_settings, eff_mu = NULL,
                           eff_var = NULL, chains_mu = NULL,
                           chains_var = NULL, prev_ll,
                           parameters, model = NULL, stage,
-                          type, tune)
+                          type, tune, r_cores = 1)
 {
   group_pars <- get_group_level(parameters, s, type)
   unq_components <- unique(tune$components)
@@ -330,9 +335,11 @@ new_particle <- function (s, data, pm_settings, eff_mu = NULL,
 
     # Calculate likelihoods
     if(tune$components[length(tune$components)] > 1){
-      lw <- calc_ll_manager(proposals[,is_shared], dadm = data, model, component = shared_idx)
+      lw <- calc_ll_manager(proposals[,is_shared], dadm = data, model,
+                            component = shared_idx, r_cores = r_cores)
     } else{
-      lw <- calc_ll_manager(proposals[,is_shared], dadm = data, model)
+      lw <- calc_ll_manager(proposals[,is_shared], dadm = data, model,
+                            r_cores = r_cores)
     }
     lw_total <- lw + prev_ll - lw[1] # make sure lls from other components are included
     # Prior density
@@ -662,13 +669,16 @@ check_prop_performance <- function(prop_performance, stage){
   return(round(prop_performance))
 }
 
-calc_ll_manager <- function(proposals, dadm, model, component = NULL){
+calc_ll_manager <- function(proposals, dadm, model, component = NULL, r_cores = 1){
   if(!is.data.frame(dadm)){
     lls <- log_likelihood_joint(proposals, dadm, model, component)
   } else{
     model <- model()
     if(is.null(model$c_name)){ # use the R implementation
-      lls <- apply(proposals,1, calc_ll_R, model, dadm = dadm)
+      lls <- unlist(
+        auto_mclapply(1:nrow(proposals),
+          function(i) calc_ll_R(proposals[i,], model=model, dadm = dadm),
+         mc.cores=r_cores))
     } else{
       p_types <- names(model$p_types)
       designs <- list()
@@ -677,9 +687,17 @@ calc_ll_manager <- function(proposals, dadm, model, component = NULL){
       }
       constants <- attr(dadm, "constants")
       if(is.null(constants)) constants <- NA
-      lls <- calc_ll(proposals, dadm, constants = constants, designs = designs, type = model$c_name,
+      if (nrow(proposals) <= r_cores)
+        lls <- calc_ll(proposals, dadm, constants = constants, designs = designs, type = model$c_name,
                      model$bound, model$transform, model$pre_transform, p_types = p_types, min_ll = log(1e-10),
-                     model$trend)
+                     model$trend) else {
+        idx <- rep(1:r_cores,each=1+(nrow(proposals) %/% r_cores))[1:nrow(proposals)]
+        lls <- unlist(auto_mclapply(1:r_cores,function(i) {
+          calc_ll(proposals[idx==i,,drop=FALSE], dadm, constants = constants,
+            designs = designs, type = model$c_name, model$bound, model$transform,
+            model$pre_transform, p_types = p_types, min_ll = log(1e-10),model$trend)
+          },mc.cores=r_cores))
+      }
     }
   }
   return(lls)
@@ -700,7 +718,20 @@ merge_group_level <- function(tmu, tmu_nuis, tvar, tvar_nuis, is_nuisance, subj_
 }
 
 
-run_hyper <- function(type, data, prior = NULL, iter = 1000, n_chains =3, ...){
+#' Run a Group-level Model.
+#'
+#' Separate function for running only the group-level model. This can be useful in a
+#' two-step analysis. Works similar in functionality to make_emc,
+#' except also does the fitting and returns an emc object that works with
+#' most posterior checking tests (but not the data generation/posterior predictives).
+#'
+#' @param prior an emc.prior object.
+#' @param iter Number of MCMC samples to collect.
+#' @inheritParams make_emc
+#'
+#' @returns an emc object with only group-level samples
+#' @export
+run_hyper <- function(type = "standard", data, prior = NULL, iter = 1000, n_chains =3, ...){
   args <- list(...)
   if(length(dim(data)) == 3){
     data_input <- data
@@ -753,3 +784,26 @@ run_hyper <- function(type, data, prior = NULL, iter = 1000, n_chains =3, ...){
   emc <- subset(emc, filter = 1)
   return(emc)
 }
+
+check_CR <- function(emc, p_vector, range = .2, N = 500){
+  covs <- diag(length(p_vector)) * range
+  props <- mvtnorm::rmvnorm(N, mean = p_vector, sigma = covs)
+  model <- emc[[1]]$model
+  if(is.null(model()$c_name)) stop("C not implemented yet for this model")
+  dat <- emc[[1]]$data[[1]]
+  modelRlist <- model()
+  modelRlist$c_name <- NULL
+  modelR <- function()return(modelRlist)
+  t1 <- system.time(
+    R <- calc_ll_manager(props, dat, modelR)
+  )
+  t2 <- system.time(
+    C <- calc_ll_manager(props, dat, model)
+  )
+  print(paste0("C ", t1$elapsed/t2$elapsed, " times faster"))
+  if(!identical(C, R)){
+    warning("C and R results differ")
+  }
+  return(list(C = C, R = R))
+}
+
