@@ -71,7 +71,7 @@ get_prior_diag_gamma <- function(prior = NULL, n_pars = NULL, sample = TRUE, N =
       if(selection != "alpha") samples$theta_var <- vars
     }
     if(selection %in% "alpha"){
-      samples$alpha <- get_alphas(mu, vars, "alpha")
+      samples$alpha <- get_alphas(mu, vars, design[[1]]$Ffactors$subjects)
     }
     out <- samples
   }
