@@ -44,7 +44,7 @@ des <- design_fmri(design_matrices)
 
 # Test whether prior plotting works
 test_that("prior_fmri", {
-  vdiffr::expect_doppelganger("prior_fmri",plot(prior(des),map=FALSE))
+  vdiffr::expect_doppelganger("prior_fmri",plot(prior(des, type = "single"),map=TRUE, N = 1e2))
 })
 
 # Test whether sampling works
