@@ -7,7 +7,7 @@ dat$covariate <- 1:nrow(forstmann)
 des <- design(data = dat, formula = list(v ~ covariate*E, B ~ E, t0 ~ S),
               model = LBA)
 
-test_that("map", {
+test_that("mapped_pars", {
   expect_snapshot(mapped_pars(des))
   expect_snapshot(mapped_pars(des, p_vector= rnorm(length(sampled_pars(des)))))
   expect_snapshot(mapped_pars(prior(des, mu_mean = c('v_covariate'  = 1))))
@@ -16,3 +16,8 @@ test_that("map", {
   expect_snapshot(mapped_pars(get_design(samples_LNR)))
 })
 
+test_that("map", {
+  expect_snapshot(credint(samples_LNR, selection = "mu", map = "E"))
+  expect_snapshot(credint(samples_LNR, selection = "mu", map = list(~ E*S)))
+  expect_snapshot(credint(samples_LNR, selection = "mu", map = TRUE))
+})
